==> clonable-clonable-magento2-1.1.0-5b4429.zip: Model/Service/ClonableTranslatorApi.php <==
<?php

namespace Clonable\Translator\Model\Service;

use Exception;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Store\Model\Store;
use Clonable\Translator\Api\Service\ClonableTranslatorApiInterface;
use Clonable\Translator\Model\ConfigManager;
use Clonable\Translator\Model\Logger\Logger;

class ClonableTranslatorApi implements ClonableTranslatorApiInterface
{
    /**
     * @var Curl
     */
    private Curl $curl;

    /**
     * @param ConfigManager $configManager
     */
    private ConfigManager $configManager;

    private Logger $logger;

    public function __construct(
        Curl $curl,
        ConfigManager $configManager,
        Logger $logger
    ){
        $this->curl = $curl;
        $this->configManager = $configManager;
        $this->logger = $logger;
    }

    /**
     * Translate a URL using the external API.
     *
     * @param string $url
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateUrlKey(string $url, Store $store): string
    {
        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareUrlTranslationData($url, $store);
        $headers = $this->prepareHeaders($store);

        $this->logger->info('Request Body: ' . json_encode($data));

        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
            $this->curl->setHeaders($headers);
            $this->curl->post(ConfigManager::API_URL_TRANSLATE, json_encode($data));

            if ($this->curl->getStatus() === 200) {
                $response = $this->curl->getBody();
                return basename(parse_url(json_decode($response)->translation, PHP_URL_PATH));
            } else {
                sleep($initialDelay * pow(2, $attempt));
            }
        }

        throw new LocalizedException(__('Failed to translate URL after multiple attempts.'));
    }

    /**
     * Translate a product name using the external API.
     *
     * @param string $text
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateText(string $text, Store $store): string
    {

        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareTextTranslationData($text, $store);
        $headers = $this->prepareHeaders($store);

        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
            $this->curl->setHeaders($headers);
            $this->curl->post(ConfigManager::API_URL_TEXT, json_encode($data));

            if ($this->curl->getStatus() === 200) {
                $response = $this->curl->getBody();
                return json_decode($response)->translation;
            } else {
                sleep($initialDelay * pow(2, $attempt));
            }
        }

        throw new LocalizedException(__('Failed to translate text after multiple attempts.'));
    }

    /**
     * Prepare data for URL translation.
     *
     * @param string $url
     * @param Store $store
     * @return array
     */
    private function prepareUrlTranslationData(string $url, Store $store): array
    {
        return [
            'url' => $this->prepareUrl($url, $store),
            'clone_id' => $this->configManager->getCloneId($store->getStoreId()),
            'site_id' => $this->configManager->getSiteId($store->getStoreId()),
            'original_language' => $this->configManager->getOriginalLanguage($store->getStoreId()),
            'target_language' => $this->configManager->getTargetLanguage($store->getStoreId()),
            'create_new' => true
        ];
    }

    /**
     * Make sure the url format is build up correctly based on the settings of the clone and the store.
     * @param string $urlKey the key that needs to be translated
     * @param Store $store the store to create the translation for
     * @return string the full url that needs to be translated
     */
    private function prepareUrl(string $urlKey, Store $store): string {
        $baseUrl = $store->getBaseUrl(); // base url always ends with a slash
        $cloneSubfolder = $this->configManager->getSubfolderClone($store);

        if ($cloneSubfolder === null) {
            return $baseUrl . $urlKey; // do not add subfolder, clone does not have it.
        } elseif ($this->strEndsWith($baseUrl, $cloneSubfolder)) {
            return $baseUrl . $urlKey; // do not add subfolder, baseUrl already contains it.
        } else {
            // make sure to add the subfolder after the base url, and make sure not to add a double slash
            $baseUrlWithoutSlash = substr($baseUrl, 0, -1);
            return $baseUrlWithoutSlash . $cloneSubfolder . $urlKey;
        }
    }

    private function strEndsWith(string $haystack, string $needle): bool {
        if ($haystack === '' && $needle !== '') {
            return false;
        }
        $len = strlen($needle);
        return substr_compare($haystack, $needle, -$len, $len) === 0;
    }

    /**
     * Prepare data for product name translation.
     *
     * @param string $name
     * @param Store $store
     * @return array
     */
    private function prepareTextTranslationData(string $name, Store $store): array
    {
        return [
            "content" => $name,
            "text_only" => true,
            "editor_mode" => false,
            "clone_id" => $this->configManager->getCloneId($store->getStoreId()),
        ];
    }

    /**
     * Prepare headers for API requests.
     *
     * @param Store $store
     * @return array
     */
    private function prepareHeaders(Store $store): array
    {
        return [
            'Authorization' => 'Bearer ' . $this->configManager->decryptApiKey($this->configManager->getApiKey($store->getStoreId())),
            'Accept' => 'application/json',
            'Content-Type' => 'application/json'
        ];
    }
}

==> clonable-clonable-magento2-1.4.0-897743.zip: Model/Service/ClonableTranslatorApi.php <==
<?php

namespace Clonable\Translator\Model\Service;

use Exception;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Store\Model\Store;
use Clonable\Translator\Api\Service\ClonableTranslatorApiInterface;
use Clonable\Translator\Model\ConfigManager;
use Clonable\Translator\Model\Logger\Logger;

class ClonableTranslatorApi implements ClonableTranslatorApiInterface
{
    /**
     * @var Curl
     */
    private Curl $curl;

    /**
     * @param ConfigManager $configManager
     */
    private ConfigManager $configManager;

    private Logger $logger;

    public function __construct(
        Curl $curl,
        ConfigManager $configManager,
        Logger $logger
    ){
        $this->curl = $curl;
        $this->configManager = $configManager;
        $this->logger = $logger;
    }

    /**
     * Translate a URL using the external API.
     *
     * @param string $urlKey
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateUrlKey(string $urlKey, Store $store): string
    {
        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareUrlTranslationData($urlKey, $store);
        $headers = $this->prepareHeaders($store);

        $this->logger->info('Request Body: ' . json_encode($data));

        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
            $this->curl->setHeaders($headers);
            $this->curl->post(ConfigManager::API_URL_TRANSLATE, json_encode($data));

            if ($this->curl->getStatus() === 200) {
                $response = $this->curl->getBody();
                return basename(parse_url(json_decode($response)->translation, PHP_URL_PATH));
            } else {
                sleep($initialDelay * pow(2, $attempt));
            }
        }

        throw new LocalizedException(__('Failed to translate URL after multiple attempts.'));
    }

    /**
     * Translate a product name using the external API.
     *
     * @param string $text
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateText(string $text, Store $store): string
    {

        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareTextTranslationData($text, $store);
        $headers = $this->prepareHeaders($store);

        $status_code = 200;
        $status_message = "";
        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
            $this->curl->setHeaders($headers);
            $this->curl->post(ConfigManager::API_URL_TEXT, json_encode($data));

            $status_code = $this->curl->getStatus();
            $response = $this->curl->getBody();
            $json = json_decode($response);

            if ($this->curl->getStatus() === 200 && $json->translation_done) {
                return $json->translation;
            } else if ($this->curl->getStatus() !== 200) {
                $status_message = $json->error_description;
                sleep($initialDelay * pow(2, $attempt));
            }
        }

        if ($status_code === 200) {
            throw new LocalizedException(__("Translation failed because you have used all of the available Clonable translation credits."));
        } else {
            throw new LocalizedException(__("Translation of text failed with status code $status_code: $status_message"));
        }
    }

    /**
     * Prepare data for URL translation.
     *
     * @param string $urlKey
     * @param Store $store
     * @return array
     */
    private function prepareUrlTranslationData(string $urlKey, Store $store): array
    {
        return [
            'url' => $this->prepareUrl($urlKey, $store),
            'clone_id' => $this->configManager->getCloneId($store->getStoreId()),
            'site_id' => $this->configManager->getSiteId($store->getStoreId()),
            'original_language' => $this->configManager->getOriginalLanguage($store->getStoreId()),
            'target_language' => $this->configManager->getTargetLanguage($store->getStoreId()),
            'create_new' => true
        ];
    }

    /**
     * Make sure the url format is build up correctly based on the settings of the clone and the store.
     * @param string $urlKey the key that needs to be translated
     * @param Store $store the store to create the translation for
     * @return string the full url that needs to be translated
     */
    private function prepareUrl(string $urlKey, Store $store): string {
        $baseUrl = $store->getBaseUrl(); // base url always ends with a slash
        $cloneSubfolder = $this->configManager->getSubfolderClone($store);

        if ($cloneSubfolder === null) {
            return $baseUrl . $urlKey; // do not add subfolder, clone does not have it.
        } elseif ($this->strEndsWith($baseUrl, $cloneSubfolder)) {
            return $baseUrl . $urlKey; // do not add subfolder, baseUrl already contains it.
        } else {
            // make sure to add the subfolder after the base url, and make sure not to add a double slash
            $baseUrlWithoutSlash = substr($baseUrl, 0, -1);
            return $baseUrlWithoutSlash . $cloneSubfolder . $urlKey;
        }
    }

    private function strEndsWith(string $haystack, string $needle): bool {
        if ($haystack === '' && $needle !== '') {
            return false;
        }
        $len = strlen($needle);
        return substr_compare($haystack, $needle, -$len, $len) === 0;
    }

    /**
     * Prepare data for product name translation.
     *
     * @param string $name
     * @param Store $store
     * @return array
     */
    private function prepareTextTranslationData(string $name, Store $store): array
    {
        return [
            "content" => $name,
            "text_only" => true,
            "editor_mode" => false,
            "clone_id" => $this->configManager->getCloneId($store->getStoreId()),
        ];
    }

    /**
     * Prepare headers for API requests.
     *
     * @param Store $store
     * @return array
     */
    private function prepareHeaders(Store $store): array
    {
        return [
            'Authorization' => 'Bearer ' . $this->configManager->decryptApiKey($this->configManager->getApiKey($store->getStoreId())),
            'Accept' => 'application/json',
            'Content-Type' => 'application/json'
        ];
    }
}
